"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = require("path");
const os_1 = require("os");
const filesystem = require("fs");
// shallow copy original function implementations before we start tweaking.
const fs = Object.assign({}, filesystem);
const common_1 = require("./common");
/** @internal */
class StaticVolumeFile {
    constructor(sourcePath) {
        this.sourcePath = sourcePath;
        this.intBuffer = Buffer.alloc(common_1.INTSIZE);
        this.buf = Buffer.alloc(1024 * 16); // 16k by default.
        /** @internal */ this.index = {};
        // clone the original static fs values and set some defaults
        this.statData = Object.assign({}, fs.statSync(sourcePath), { isDirectory: () => false, isSymbolicLink: () => false, isBlockDevice: () => false, isCharacterDevice: () => false, isFile: () => false, isFIFO: () => false, isSocket: () => false, size: 0 });
        // read the index
        this.fd = fs.openSync(sourcePath, 'r');
        // close on process exit.
        let dataOffset = this.readInt();
        // read hash
        let hashSize = this.readInt();
        if (hashSize > this.buf.length) {
            this.buf = Buffer.alloc(hashSize);
        }
        this.readBuffer(this.buf, hashSize);
        this.hash = this.buf.toString('utf8', 0, hashSize);
        do {
            const nameSz = this.readInt();
            if (nameSz == 0) {
                break;
            }
            const dataSz = this.readInt();
            if (nameSz > this.buf.length) {
                this.buf = Buffer.alloc(nameSz);
            }
            this.readBuffer(this.buf, nameSz);
            const name = this.buf.toString('utf8', 0, nameSz);
            // add entry for file into index
            this.index[name] = Object.assign({}, this.statData, { ino: dataOffset, size: dataSz, blocks: 1, blksize: dataSz, isFile: () => true });
            // ensure parent path has a directory entry
            this.addParentFolders(name);
            dataOffset += dataSz;
        } while (true);
    }
    readBuffer(buffer, length) {
        return fs.readSync(this.fd, buffer, 0, length || buffer.length, null);
    }
    readInt() {
        fs.readSync(this.fd, this.intBuffer, 0, common_1.INTSIZE, null);
        return this.intBuffer.readIntBE(0, 6);
    }
    shutdown() {
        fs.closeSync(this.fd);
        this.index = {};
    }
    addParentFolders(name) {
        const parent = path_1.dirname(name);
        if (parent && !this.index[parent]) {
            this.index[parent] = Object.assign({}, this.statData, { isDirectory: () => true });
            return this.addParentFolders(parent);
        }
    }
    readFile(filepath, options) {
        const item = this.index[filepath];
        if (item && item.isFile()) {
            const encoding = options ?
                typeof options === 'string' ? options :
                    typeof options === 'object' ? options.encoding || 'utf8' : 'utf8' : 'utf8';
            // realloc if necessary
            if (this.buf.length < item.size) {
                this.buf = Buffer.alloc(item.size);
            }
            // read the content and return a string
            fs.readSync(this.fd, this.buf, 0, item.size, item.ino);
            return this.buf.toString(encoding, 0, item.size);
        }
        return undefined;
    }
}
exports.StaticVolumeFile = StaticVolumeFile;
class StaticFilesystem {
    constructor() {
        this.fileSystems = [];
    }
    NewError(code, method, filepath) {
        switch (code) {
            case os_1.constants.errno.ENOENT:
                return Object.assign({}, new Error(`ENOENT: no such file or directory, ${method} '${filepath}'`), { code: 'ENOENT', path: filepath, errno: os_1.constants.errno.ENOENT });
            case os_1.constants.errno.EISDIR:
                return Object.assign({}, new Error(`EISDIR: illegal operation on a directory, ${method} '${filepath}'`), { code: 'EISDIR', path: filepath, errno: os_1.constants.errno.EISDIR });
        }
        return Object.assign({}, new Error(`UNKNOWN: Error, ${method} '${filepath}'`), { code: 'UNKNOWN', path: filepath, errno: -10000 });
    }
    shutdown() {
        for (const fsystem of this.fileSystems) {
            fsystem.shutdown();
        }
    }
    get hashes() {
        return common_1.select(this.fileSystems, (p, c, i, a) => c.hash);
    }
    load(sourcePath) {
        sourcePath = path_1.resolve(sourcePath);
        for (let i = 0; i < this.fileSystems.length; i++) {
            if (this.fileSystems[i].sourcePath === sourcePath) {
                // already loaded?
                return this;
            }
        }
        this.fileSystems.push(new StaticVolumeFile(sourcePath));
        return this;
    }
    get loadedFileSystems() {
        return common_1.select(this.fileSystems, (p, c) => c.sourcePath);
    }
    unload(sourcePath) {
        sourcePath = path_1.resolve(sourcePath);
        for (let i = 0; i < this.fileSystems.length; i++) {
            if (this.fileSystems[i].sourcePath === sourcePath) {
                this.fileSystems[i].shutdown();
                this.fileSystems.splice(i, 1);
            }
        }
        return this;
    }
    get entries() {
        return common_1.selectMany(this.fileSystems, (p, c) => Object.keys(c.index));
    }
    readFileSync(filepath, options) {
        const targetPath = common_1.unixifyPath(filepath);
        return common_1.first(this.fileSystems, (fsystem) => fsystem.readFile(targetPath, options), () => { throw this.NewError(os_1.constants.errno.ENOENT, "readFileSync", filepath); });
    }
    realpathSync(filepath) {
        const targetPath = common_1.unixifyPath(filepath);
        return common_1.first(this.fileSystems, (fsystem) => fsystem.index[targetPath] ? targetPath : undefined, () => { throw this.NewError(os_1.constants.errno.ENOENT, "realpathSync", filepath); });
    }
    ;
    statSync(filepath) {
        const targetPath = common_1.unixifyPath(filepath);
        return common_1.first(this.fileSystems, (fsystem) => fsystem.index[targetPath], () => { throw this.NewError(os_1.constants.errno.ENOENT, "statSync", filepath); });
    }
}
exports.StaticFilesystem = StaticFilesystem;
//# sourceMappingURL=static-filesystem.js.map