#!/usr/bin/env node
"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const common_1 = require("./lib/common");
const path_1 = require("path");
const fs_1 = require("fs");
const os_1 = require("os");
const static_filesystem_1 = require("./lib/static-filesystem");
const main_1 = require("./main");
function help(reason) {
    console.log("static-link");
    console.log(reason);
}
async function yarnInstall(directory, pkgs) {
    const c = `${__dirname}/../node_modules/yarn/bin/yarn add --json ${pkgs.map((a) => `"${a}"`).join(' ')}`;
    if (debug) {
        console.log(c);
    }
    const output = await common_1.execute(c, { cwd: directory });
    if (debug) {
        console.log(output.stdout);
    }
    if (output.error) {
        throw Error(`Failed to install package '${pkgs}' -- ${output.error}`);
    }
}
async function npmInstall(directory, pkg) {
    const output = await common_1.execute(`${__dirname}/../node_modules/yarn/bin/yarn add --json "${pkg}"`, { cwd: directory });
    if (output.error) {
        throw Error(`Failed to install package '${pkg}' -- ${output.error}`);
    }
}
function getArg(arg) {
    const match = `--${arg}`;
    for (const each of process.argv.slice(2)) {
        if (each.startsWith(match)) {
            if (each === match) {
                return true;
            }
            const more = each.substr(match.length);
            switch (more.charAt(0)) {
                case ':':
                case '=':
                    return more.substr(1);
            }
        }
    }
    return undefined;
}
const debug = getArg("debug") == true;
async function main() {
    // load package.json
    const pkgfile = path_1.resolve(getArg("input") || "./package.json");
    const force = getArg("force") == true;
    if (!(await common_1.isFile(pkgfile))) {
        return help("Unknown or missing project.json file.");
    }
    const basefolder = path_1.dirname(pkgfile);
    const restorePkgFile = await common_1.backup(pkgfile);
    const restorePkgLock = await common_1.backup(path_1.join(path_1.dirname(pkgfile), "package-lock.json"));
    const restoreYarnLock = await common_1.backup(path_1.join(path_1.dirname(pkgfile), "yarn.lock"));
    const restoreYarnIntegrity = await common_1.backup(path_1.join(path_1.dirname(pkgfile), "node_modules", ".yarn-integrity"));
    const minorTasks = new Array();
    const workingdir = fs_1.mkdtempSync(path_1.join(os_1.tmpdir(), "static_modules"));
    console.log(workingdir);
    try {
        const pkgJson = require(pkgfile);
        const config = pkgJson["static-link"];
        if (!config) {
            return help("no 'static-link' section in package.json.");
        }
        config.filesystem = path_1.resolve(config.filesystem || `${basefolder}/dist/static_modules.fs`);
        config.loader = config.loader || `${basefolder}/dist/static-loader.js`;
        if (!config.entrypoints) {
            config.entrypoints = new Array();
            if (pkgJson.main) {
                if (typeof pkgJson.main === 'string') {
                    config.entrypoints.push(pkgJson.main);
                }
                else {
                    try {
                        if ((await common_1.stat(`${basefolder}/index.js`)).isFile()) {
                            config.entrypoints.push(`${basefolder}/index.js`);
                        }
                    }
                    catch (_a) { }
                }
            }
            if (pkgJson.bin) {
                if (typeof pkgJson.bin === 'string') {
                    if (config.entrypoints.indexOf(pkgJson.bin) == -1) {
                        config.entrypoints.push(pkgJson.bin);
                    }
                }
                else {
                    for (const ep in pkgJson.bin) {
                        const entrypoint = pkgJson.bin[ep];
                        if (config.entrypoints.indexOf(entrypoint) == -1) {
                            config.entrypoints.push(entrypoint);
                        }
                    }
                }
            }
        }
        if (!config.dependencies) {
            return help("packages to static link should be in 'static-link/dependencies' section");
        }
        // we got this far. looks like we should be able to do this.
        // copy the static loader into place. always a good idea.
        config.loader = path_1.resolve(basefolder, config.loader);
        const sourcefile = `${__dirname}/lib/static-loader.js`;
        let doCopy = force;
        try {
            doCopy = (await common_1.stat(sourcefile)).size !== (await common_1.stat(config.loader)).size;
        }
        catch (_b) {
            doCopy = true;
        }
        if (doCopy) {
            console.log(`> Writing static loader: ${config.loader}`);
            minorTasks.push(common_1.copyFile(sourcefile, config.loader));
        }
        // patch the entrypoints
        for (const each of config.entrypoints) {
            const entrypoint = path_1.resolve(basefolder, each);
            if (await common_1.isFile(entrypoint)) {
                let loaderPath = path_1.relative(path_1.dirname(entrypoint), config.loader).replace(/\\/g, "/");
                if (loaderPath.charAt(0) != '.') {
                    loaderPath = `./${loaderPath}`;
                }
                let fsPath = path_1.relative(path_1.dirname(entrypoint), config.filesystem).replace(/\\/g, "/");
                fsPath = `\${__dirname }/${fsPath}`;
                let content = await common_1.readFile(entrypoint, { encoding: "utf8" });
                const patchLine = `require('${loaderPath}').load(\`${fsPath}\`)\n`;
                let prefix = "";
                if (content.indexOf(patchLine) == -1 || force) {
                    const rx = /^#!.*$/gm.exec(content);
                    if (rx && rx.index == 0) {
                        prefix = `${rx[0]}\n`;
                        // remove prefix
                        content = content.replace(prefix, "");
                    }
                    // strip existing loader
                    content = content.replace(/^require.*static-loader.js.*$/gm, "");
                    content = content.replace(/\/\/ load static module: .*$/gm, "");
                    content = content.trim();
                    content = `${prefix}// load static module: ${fsPath}\n${patchLine}\n${content}`;
                    console.log(`> Patching Entrypoint: ${entrypoint}`);
                    await common_1.writeFile(entrypoint, content);
                }
            }
        }
        // if the fs exists, check the hash to see if we need to do this.
        const hash = common_1.calculateHash(config);
        try {
            if (common_1.isFile(config.filesystem)) {
                if (!force) {
                    const svf = new static_filesystem_1.StaticVolumeFile(config.filesystem);
                    svf.shutdown();
                    if (svf.hash === hash) {
                        // hash matches the file
                        console.log("File hash matches configuration hash: Skipping generating file system.");
                        return;
                    }
                }
                fs_1.unlinkSync(config.filesystem);
            }
        }
        catch (_c) {
            // doens't appear so.
        }
        // install packages
        try {
            const all = new Array();
            const set = new Array();
            for (const pkg in config.dependencies) {
                const version = config.dependencies[pkg];
                console.log(`> Installing static dependency ${pkg}@${version} `);
                set.push(`${pkg}@${version}`);
            }
            all.push(yarnInstall(workingdir, set));
            all.push(yarnInstall(basefolder, set));
            await Promise.all(all);
            // we've installed everything
            // let's pack up the file
            const sf = new main_1.StaticFilesystemCreator();
            await sf.addFolder(`${workingdir}/node_modules`, "/node_modules");
            console.log(`> Writing filesystem: ${config.filesystem}`);
            await common_1.mkdir(path_1.dirname(config.filesystem));
            await sf.write(config.filesystem, hash);
        }
        finally {
        }
    }
    catch (err) {
        return help(err);
    }
    finally {
        await Promise.all(minorTasks);
        await restorePkgFile();
        await restorePkgLock();
        await restoreYarnLock();
        await restoreYarnIntegrity();
        if (workingdir) {
            await common_1.rmdir(workingdir);
        }
    }
}
main();
//# sourceMappingURL=static-link.js.map